##########################################################################
# imfit_test.py
#
# Copyright (C) 2008, 2009
# Associated Universities, Inc. Washington DC, USA.
#
# This script is free software; you can redistribute it and/or modify it
# under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
# License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 675 Massachusetts Ave, Cambridge, MA 02139, USA.
#
# Correspondence concerning AIPS++ should be adressed as follows:
#        Internet email: aips2-request@nrao.edu.
#        Postal address: AIPS++ Project Office
#                        National Radio Astronomy Observatory
#                        520 Edgemont Road
#                        Charlottesville, VA 22903-2475 USA
#
# <author>
# Dave Mehringer
# </author>
#
# <summary>
# Test suite for the CASA task imcollapse
# </summary>
#
# <reviewed reviwer="" date="" tests="" demos="">
# </reviewed
#
# <prerequisite>
# <ul>
#   <li> <linkto class="task_imcollapse.py:description">imcollapse</linkto> 
# </ul>
# </prerequisite>
#
# <etymology>
# Test for the imcollapse task
# </etymology>
#
# <synopsis>
# Test the imcollapse task and the ia.collapse() method upon which it is built.
# </synopsis> 
#
# <example>
#
# This test runs as part of the CASA python unit test suite and can be run from
# the command line via eg
# 
# `echo $CASAPATH/bin/casa | sed -e 's$ $/$'` --nologger --log2term -c `echo $CASAPATH | awk '{print $1}'`/code/xmlcasa/scripts/regressions/admin/runUnitTest.py test_imcollapse[test1,test2,...]
#
# </example>
#
# <motivation>
# To provide a test standard for the imcollapse task to ensure
# coding changes do not break the associated bits 
# </motivation>
#

###########################################################################
import shutil
import casac
from tasks import *
from taskinit import *
from __main__ import *
import unittest

class ia_convolve_test(unittest.TestCase):
    
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def test_stretch(self):
        """ ia.convolve(): Test stretch parameter"""
        yy = iatool()
        mymask = "maskim"
        yy.fromshape(mymask, [200, 200, 1, 1])
        yy.addnoise()
        yy.done()
        kernel = "kernel"
        shape = [200,200,1,20]
        yy.fromshape(kernel, shape)
        yy.fromshape("", shape)
        yy.addnoise()
        yy.done()
        yy.fromshape("", shape)
        self.assertRaises(
            Exception,
            yy.convolve, "", mymask, mask=mymask + ">0", stretch=False
        )
        zz = yy.convolve("", mymask, mask=mymask + ">0", stretch=True)
        self.assertTrue(type(zz) == type(yy))
        yy.done()
        zz.done()

    def test_history(self):
        """Test history writing"""
        yy = iatool()
        kernel = "khistory"
        shape = [200,200,1,20]
        for mytype in ('f', 'c', 'd', 'cd'):
            yy.fromshape(kernel, shape, overwrite=True, type=mytype)
            yy.fromshape("", shape)
            yy.addnoise()
            yy.fromshape("", shape, type=mytype)
            if mytype == 'c' or mytype == 'cd':
                self.assertRaises(Exception, yy.convolve, "", kernel)
                continue
            zz = yy.convolve("", kernel)
            yy.done()
            msgs = zz.history()
            zz.done()
            self.assertTrue("convolve" in msgs[-2])
            self.assertTrue("convolve" in msgs[-1])
        
def suite():
    return [ia_convolve_test]
